/*
 * MAX : bttv.cc, Wed Jan  2 14:44:59 CET 2002 -sg
 *
 * Device driver for CCD camera reading of data (via bttv, videodev) 
 * 	- experimental version !
 *
 * This file is part of Max data acquisition software
 * (c) 2001 by S. Gerlach
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>
#include "/usr/local/Linux-Lab/bttv/driver/videodev.h"
#include "list.hh"
#include "sched.hh"
#include "camacs.h"
#include "error.hh"


#define DEBUG_DEV_BTTV
	
int width = 640;
int height = 480;
int format = VIDEO_PALETTE_RGB24;

int grab;

// global variables
int  v;
struct video_mbuf vmbuf;
struct video_mmap map;
unsigned char *m;

/******************* function(s) common to all channels **********************/
int bttv_grab (Scheduler *parameters )
{
	int i,j,sum=0;

	ioctl(v,VIDIOCMCAPTURE,&map);
        ioctl(v,VIDIOCSYNC,&map.frame);
	
	m = (unsigned char *) mmap(0, vmbuf.size, PROT_READ, MAP_PRIVATE, v, 0);
	if ( (int)m <= 0 ) {
		perror("mmap failed");
		exit(1);
	}

	// create ppm-file on stdout
	/* TODO
	if (grab) {
	printf("P3\n%d %d\n255\n",width,height);
	  for (j = 0;j<height;j++) {
	  for (i = 0;i<width;i++) {
	  printf("%d %d %d ",m[3*(i+j*width)],
	  m[3*(i+j*width)]+1,m[3*(i+j*width)+2]);
	  }
	  printf("\n");
	  }
	}*/ 

	// height = 250-380, width = 320-450
	for (j = 250;j<380;j++) {
		for (i = 320;i<450;i++) {
			sum += m[3*(i+j*width)];
		}
	}

	parameters->store(sum);
	munmap((char *) m,vmbuf.size);

#ifdef DEBUG_DEV_BTTV
  printf("sum : \"%d\" done\n",sum);
#endif

  return 1;
}
/*****************************************************************************/

//init function
//called each time before meausurement starts
int bttv_init(List *device_data, Scheduler *parameters)
{	
	struct video_capability vcap;
        struct video_channel vch;

  	printf("\nInitializing bttv");

	v = open("/dev/bttv", O_RDWR);
        if ( v < 0) {
                perror("open failed");
                exit(1);
        }

	// Video Capabilities
        if ( ioctl(v, VIDIOCGCAP, &vcap) < 0) {
                perror("VIDIOCGCAP failed");
                exit(1);
        }

#ifdef DEBUG_DEV_BTTV
        fprintf(stderr,"opened /dev/bttv\nname = %s\ntype = %d\nchannels = %d\n
audios = %d\n",
                vcap.name,vcap.type,vcap.channels,vcap.audios);
        fprintf(stderr,"max/min width = %d %d\n",vcap.maxwidth,vcap.minwidth);
        fprintf(stderr,"max/min height = %d %d\n",vcap.maxheight,vcap.minheight)
;
#endif   

	//Setting channnel
        vch.channel = 1;
        vch.norm = VIDEO_MODE_AUTO;
        //vch.type=VIDEO_TYPE_CAMERA;
        ioctl(v,VIDIOCSCHAN,&vch);

#ifdef DEBUG_VIDEO
        fprintf(stderr,"setting channel to Composite1\n");
#endif

	if ( ioctl(v, VIDIOCGMBUF, &vmbuf) < 0) {
                perror("VIDIOCGMBUF failed");
                exit(1);
        }

        map.width=width;
        map.height=height;
        map.format=format;
        map.frame=0;

	device_data->getvar(&grab,"bttv","grab");

  	return 1;
}

int bttv_pre ( List *device_data, Scheduler *parameters )
{
  // nothing to do...

  return 1;
}

int bttv ( List *device_data, Scheduler *parameters )
{
      	if ( ! bttv_grab(parameters))
		return 0;
  	
	return 1;
}

int bttv_close(List *device_data, Scheduler *parameters)
{
  printf("\nclosing bttv ");
	close(v);

  return 1; //function has to return a value -> 1 otherwise measurement stops
}
