/*
 * MAX : io612.cc, Wed Jan  2 14:44:59 CET 2002 -sg
 *
 * Device driver for the DSP 612 Dual 24 bit I/O Register
 *
 * This file is part of Max data acquisition software
 * (c) 2001 S. Gerlach
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include "list.hh"
#include "sched.hh"
#include "camacs.h"
#include "error.hh"

//#define DEBUG_DEV_IO612

// global variables
static int  N, qx, bit;

int io612_setbit ( int bit, Scheduler *parameters )
{
	unsigned int bitvalue = (int) pow(2,bit-1);

	qx = camac(16,0,N,&bitvalue);   //set bit

#ifdef DEBUG_DEV_IO612
	printf("\nio612: set bit %d (%d)\n",bit,bitvalue);
#endif

	return 1;
}

int io612_unsetbit (Scheduler *parameters )
{
	unsigned int bitvalue =0;

	camac(16,0,N,&bitvalue);   //set bit

#ifdef DEBUG_DEV_IO612
	printf("\nio612: unset all bits \n");
#endif

	return 1;
}

/*****************************************************************************/

//init function
//called each time before meausurement starts
int io612_init(List *device_data, Scheduler *parameters)
{	
  if (camac_init() != 1)
    if (!parameters->error(E_CAMAC,"Couldn't initialize device "))
      return 0;

  printf("\ninitializing io612\n");
  device_data->getvar(&N,"io612","N");
  device_data->getvar(&bit,"io612","bit");

  return 1;
}

int io612_pre ( List *device_data, Scheduler *parameters )
{
		
	if (parameters->act_channel() % 2)
		io612_setbit(bit,parameters);
	else
		io612_unsetbit(parameters);
		
	return 1;
}

int io612 ( List *device_data, Scheduler *parameters )
{
	//nothing
	return 1;
}

int io612_close(List *device_data, Scheduler *parameters)
{
	printf("\nclosing io612 ");
	
	if (camac_close() != 1)
		if (!parameters->error(E_CAMAC,"Couldn't close device"))
			return 0;

	return 1; //function has to return a value -> 1 otherwise measurement stops
}

