/*
 * MAX : le2323.cc, Fri Feb 15 18:11:21 CET 2002 -sg
 *
 * Device driver for the LeCroy 2323 Programmable Dual Gate Generator
 *
 * This file is part of Max data acquisition software
 * (c) 2002 S. Gerlach
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include "list.hh"
#include "sched.hh"
#include "camacs.h"
#include "error.hh"

#define DEBUG_DEV_LE2323

// global variables
static int  N, width, delay, start_delay, end_delay;

void set_word(int w, int d) {
	int c=0, i=0, l=0; 
	unsigned int word=0;

	switch(w) {
	case 10:
		w=0;
		break;
	case 30:
		w=1;
		break;
	case 100:	
		w=2;
		break;
	case 300:
		w=3;
		break;
	default:
		printf("LE2323 : False Width! Setting to 100 ns\n");
		w=0;
		break;
	}

	if (d < 50 || d > 10.0e10)
		printf("LE2323 : False Delay\n");
	else if (d < 100)
		printf("LE2323 : reduced accurency and stability\n");

	while (i++ < 8) {
		if (d <= 1000)
			break;
		else {
			d /= 10;
			c +=1;
		}
	}

	// 
	c = c << 10;
	word = c | d;
	l = l << 13;
	word = l | word;
	w = w << 14;
	word = w | word;

	camac(17,0,N,&word);

#ifdef DEBUG_DEV_LE2323
	//printf("Sending word %d\n",word);
	
	//camac(1,0,N,&word);
	//printf("Le2323 says : word = %d\n",word);
#endif
}

/*****************************************************************************/

//init function
//called each time before meausurement starts
int le2323_init(List *device_data, Scheduler *parameters)
{	
  	if (camac_init() != 1)
    		if (!parameters->error(E_CAMAC,"Couldn't initialize device"))
      			return 0;

  	printf("LE2323 : Initializing\n");
  	device_data->getvar(&N,"le2323","N");
  	device_data->getvar(&width,"le2323","width");
  	device_data->getvar(&delay,"le2323","delay");
  	device_data->getvar(&start_delay,"le2323","start_delay");
  	device_data->getvar(&end_delay,"le2323","end_delay");

	// Setze width & delay
	set_word(width,delay);

  return 1;
}

int le2323_pre ( List *device_data, Scheduler *parameters )
{
	int channel = parameters->act_channel();
	int chanmax = parameters->maximal_channels();

	if (chanmax >1) {
		delay = start_delay + 
			(channel-1)*(end_delay-start_delay)/(chanmax-1);
	}
	else {
		delay = start_delay;
	}

	// set delay
	set_word(width,delay);

# ifdef DEBUG_DEV_LE2323
	printf("LE2323 : Setting Delay to %d ns\n",delay);
# endif

	device_data->putvar(delay,"le2323","delay");

	return 1;
}

int le2323 ( List *device_data, Scheduler *parameters )
{
	//nothing
	return 1;
}

int le2323_close(List *device_data, Scheduler *parameters)
{
	printf("LE2323 : Closing\n");
	
	device_data->putvar(delay,"le2323","delay");
	
	if (camac_close() != 1)
		if (!parameters->error(E_CAMAC,"Couldn't close device"))
			return 0;

	return 1;
}
