/*
 * MAX : camacs.cc, Sam Dez 29 19:24:57 CET 2001
 * 
 * CAMAC interface for use with the MAX data aquisition program
 *
 *
 *    11.12.1988  M.Voelkel
 *    31.01.1994  D.Rohloff    geaendert fuer microsys ieee
 *    04.01.1996  J.Ludwig     Programmierstil geaendert
 *    10.07.1997  C.Otto       fr Linux umgeschrieben
 *    08.04.1998  M. Dammasch  auf mehrere IEEE-Devices verallgemeinert
 *    16.08.2002  S. Gerlach	cleared programming style
 *    02.09.2002  S. Gerlach	unsigned char daten[4] !!!!!!!!!	
 *    23.09.2002  S. Gerlach	_CAMAC_GPIB_NEW_
 *
 * - CAMAC Functions for the LeCroy 8901 GPIB Crate controller: 
 * 		_CAMAC_GPIB_		(Linux-GPIB Driver)
 		_CAMAC_INES_GPIB_	(INES-GPIB Driver)
 * - CAMAC Functions for the Wiener PC16 CAMAC Crate controller: _CAMAC_C111A_
 *   This driver is based on the c111a driver for the Wiener PC16Turbo card
 *   (c) Giovanni Organtini, organtini@roma1.infn.it
 */

#include <stdio.h>
#include <stdlib.h>

#ifdef _CAMAC_GPIB_
#include <ib.h>
#include "gpiberr.h"
#endif

#ifdef _CAMAC_C111A_
#include <c111a.h>
#include <naf.h>
#endif

#include "camacs.h"

//#define DEBUG_CAMAC

/*
 * global variables
 */
#ifdef _CAMAC_GPIB_
int  Eid_Cam;  // GPIB-Address of LeCroy-CAMAC-Controller
#endif

#ifdef _CAMAC_C111A_
int  cc16_fd;  // file descriptor of PC16 C111A-CAMAC-Controller
#endif
int  CamInit;  // CAMAC initialization flag


// CAMAC_INIT
int  camac_init(void) {
#ifdef _CAMAC_GPIB_
	char d[5];
	char buf;
#endif
#ifdef _CAMAC_C111A_
	int q, x, data;
#endif

	if (CamInit) // exit if already initialized
 		return 1;

	CamInit = 0; // set CAMAC not initialized

#ifdef _CAMAC_GPIB_
  	printf("\n***** Connecting GPIB Device 'LeCroy' *****\n");

 	if((Eid_Cam = ibfind("LeCroy")) & ERR){
    		gpiberr("ibfind error");
    		return 0;
  	}

#ifdef DEBUG_CAMAC
  	printf("LeCroy : GPIB-Address is set to #%d\n",Eid_Cam);    

	#ifndef _CAMAC_GPIB_NEW_ 
  		printf("LeCroy : Send IFC\n");  // ibsic aus Lib.
	#endif
#endif
		
#ifndef _CAMAC_GPIB_NEW_ 
  	if( ibsic(Eid_Cam) & ERR){
    		gpiberr("ibsic err");
    	return 0;
  	}

	#ifdef DEBUG_CAMAC
 	printf("LeCroy : Send REM\n");  // ibsre aus Lib.
	#endif
  	if( ibsre(Eid_Cam,1) & ERR){
    		gpiberr("ibsre err");
    		return 0;
  	}

#endif
	
#ifdef DEBUG_CAMAC
  	printf("LeCroy : Clearing GPIB bus\n");  // ibclr aus Lib.
#endif
  	if( ibclr(Eid_Cam) & ERR){
    		gpiberr("ibclear error");
    		return 0;
  	}
#endif	// CAMAC_GPIB
  
#ifdef _CAMAC_C111A_
  	printf("\n\n***** Connecting CAMAC-Controller 'C111A' ***** ");
  
  	if ((cc16_fd = c111aopen(C111A_ADDR)) <= 0) {  // failed
    		switch (cc16_fd) {
    		case -2 :
      			fprintf(stderr,"\nCamac_Init(): Couldn't get file handle for C111A@%d ",C111A_ADDR); 
      			break;
    		case -1 :
      			fprintf(stderr,"\nCamac_Init(): Invalid crate address (%d) for C111A ",C111A_ADDR); 
      			break;
    		default :
      			fprintf(stderr,"\nCamac_Init(): Illegal file descriptor (%d) for C111A@%d ",cc16_fd,C111A_ADDR); 
      			break;
    		}
    		return 0;
  	}
  	printf("\n'C111A' file handle is set to #%d ",cc16_fd);
#endif
  
	/****** Init Crate Controller *******/
#ifdef _CAMAC_GPIB_
	#ifndef _CAMAC_GPIB_NEW_
	#ifdef DEBUG_CAMAC
  		printf("LeCroy : Send CAMAC-Clear(C) and CAMAC-Initialize(Z)\n");
	#endif
	buf='\x23';
  	if( ibwrt (Eid_Cam,&buf,1)& ERR ){  // Send both CLEAR and INITIALIZE
    		gpiberr("write error");
    		return 0;
  	}
  	if( ibrd (Eid_Cam,d,4)& ERR){
    		gpiberr("read error");
    		return 0;	
  	}
  
#ifdef DEBUG_CAMAC
  	printf("LeCroy : Set data mode: ");
#endif
	buf='\x64';
  	if( ibwrt (Eid_Cam,&buf,1)& ERR ){  // set data transfer mode (24bit)
    		gpiberr("write error");
    	return 0;
  	}
  	if( ibrd (Eid_Cam,d,4)  & ERR){
    		gpiberr("read error");
    		return 0;
  	}
	#ifdef DEBUG_CAMAC
  	printf("24-bit read normal transfer\n");
	#endif
	#endif
#endif

#ifdef _CAMAC_C111A_
  	q = x = data = 0;
  	printf("\nC111A_%d.25: F(36)A(1)[%d]: Clear I, C cycle ",cc16_fd,data);
 	c111a(cc16_fd, 25, 1, 36, &q, &x, &data);
  	if (x == 1)  // okay
    		printf("done\n");
  	else {
    		printf("failed (QX: %d <%d%d>\t D: %d (0x%x)) ",2*q+x,q,x,data,data); 
    		fprintf(stderr,"\nCamac_Init(): C111A_%d.25: Couldn't execute F(36)A(1) ",cc16_fd);
    		return 0;
  	}

  	q = x = data = 0;
  	printf("\nC111A_%d.25: F(38)A(1)[%d]: Clear I, Z, C cycle ",cc16_fd,data);
  	c111a(cc16_fd, 25, 1, 38, &q, &x, &data);
  	if (x == 1)  // okay
    		printf("done\n");
  	else {
    		printf("failed (QX: %d <%d%d>\t D: %d (0x%x)) ",2*q+x,q,x,data,data); 
    		fprintf(stderr,"\nCamac_Init(): C111A_%d.25: Couldn't execute F(38)A(1) ",cc16_fd);
    		return 0;
  	}
#endif

  	CamInit = 1;
  	printf("\n***** Completed Successfully *****\n\n");

  	return 1;
 }


int camac_close () {
#ifdef _CAMAC_GPIB_
  	char d[5];
	char buf;
#endif
#ifdef _CAMAC_C111A_
  	int q, x, data;
#endif
  
  	if (CamInit) { 
#ifdef _CAMAC_GPIB_
#ifndef _CAMAC_GPIB_NEW_
#ifdef DEBUG_CAMAC
    		printf("LeCroy : Send CAMAC-Clear(C) ...\n");
#endif
		buf='\x22';
    		if( ibwrt (Eid_Cam,&buf,1)& ERR ){  // Send CLEAR 
      			gpiberr("write error");
      			return 0;
    		}

    		if( ibrd (Eid_Cam,d,4)& ERR){
 			gpiberr("read error");
     			return 0;	
    		}
#endif
#ifdef DEBUG_CAMAC
    		printf("LeCroy : Clearing GPIB bus\n");  // ibclr aus Lib.
#endif
    		if( ibclr(Eid_Cam) & ERR){
      			gpiberr("ibclear error");
      			return 0;
    		}
#endif
    
#ifdef _CAMAC_C111A_
    		q = x = data = 0;
    		printf("\nC111A_%d.25: F(38)A(1)[0]: Clear I, Z, C cycle ",cc16_fd);
    		c111a(cc16_fd, 25, 1, 38, &q, &x, &data);
    		if (x == 1)  // okay
      			printf("done\n");
    		else {
      			printf("failed (QX: %d <%d%d>\t D: %d (0x%x)) ",2*q+x,q,x,data,data); 
      			fprintf(stderr,"\nCamac_Close(): C111A_%d.25: Couldn't execute F(38)A(1) ",cc16_fd);
      			return 0;
    		}
    
    		c111aclose(cc16_fd);
#endif
  	}
  
  	CamInit = 0;

	ibonl(Eid_Cam,0);
	
  	return 1;
}


int camac(int f,int a,int n,unsigned int *d) {
#ifdef _CAMAC_GPIB_
	unsigned char camac_befehl[6];		/* Befehlsstring fuer CAMAC Aufruf */
  	unsigned char camac_daten[4];		/* Datenstring fuer CAMAC Antwort */
  	int anzahl_bytes;		/* Anzahl der Bytes fuer Uebertragung */

  	if (f > CAMAC_BEFEHL) {		/* CAMAC Befehl ungueltig */
      		printf("Wrong CAMAC setting due to\n");
      		exit(0);
    	}
  	else {
      		camac_befehl[CAMAC_F] = (char) (f & 0xff);     /* Unteres Byte von f in Befehlsstring */
      		camac_befehl[CAMAC_N] = (char) (n & 0xff);     /* Unteres Byte von n in Befehlsstring */
      		camac_befehl[CAMAC_A] = (char) (a & 0xff);     /* Unteres Byte von a in Befehlsstring */
      
	      	switch (f & CAMAC_BEFEHLS_MASKE) {
		case CAMAC_READ:
		case CAMAC_SET_STATUS_A:
		case CAMAC_SET_STATUS_B:
		  	anzahl_bytes = 3;
		  	break;
		case CAMAC_WRITE:
		  	camac_befehl[CAMAC_D1] = (char) (*d & 0xff);
		 	camac_befehl[CAMAC_D2] = (char) ((*d & 0xff00 ) >> 8);
		  	camac_befehl[CAMAC_D3] = (char) ((*d & 0xff0000 ) >> 16);
		  	anzahl_bytes = 6;
		  	break;
		default:
		  	anzahl_bytes = 0;
		  	printf("Wrong CAMAC setting due to\n");
		  	exit(0);
		  	break;
		}
      		if( ibwrt(Eid_Cam, camac_befehl, anzahl_bytes) & ERR ){
			printf("\n Warning, error occured!");
			gpiberr("write error");
			exit(1);
      		}
     	 
      		if( ibrd (Eid_Cam, camac_daten, 4) & ERR){
			printf("\n Warning, error occured!");
			gpiberr("read error");
			exit(1);
      		}

#ifdef DEBUG_CAMAC
		printf("\nCAMAC@%02d.%02d: F(%02d)A(%02d)[%d] -> QX: %d   D: %d ",Eid_Cam,n,f,a,*d,
			(int)camac_daten[3],(int) (camac_daten[0] | ( camac_daten[1] << 8 ) | ( camac_daten[2] << 16 )));
#endif
             
		if ((f & CAMAC_BEFEHLS_MASKE) == CAMAC_READ) {	/* Daten zusaetzlich zu QX wurden gelesen */
 			*d=camac_daten[0] | ( camac_daten[1] << 8 ) | ( camac_daten[2] << 16 );
		}
      		return ( (int) camac_daten[3]);   /* Rckgabewert: QX (=3 bei Erfolg)*/
	}
#endif

	
      // C111A	
#ifdef _CAMAC_C111A_
  	int q=0, x=0;
  
#ifdef DEBUG_CAMAC
  	printf("\nC111A_%d.%02d: F(%d)A(%d)[%d] ",cc16_fd,n,f,a,*d);
#endif      
  
  	c111a(cc16_fd, n, a, f, &q, &x,(int *) d);
  
#ifdef DEBUG_CAMAC
  	printf("-> QX: %d <%d%d>\t D: %d (0x%x) ",2*q+x,q,x,*d,*d);
#endif               
  
  	return(2*q+x);
#endif
}
