/*
 * MAX : list.cc, Sam Dez 29 19:48:02 CET 2001 -sg
 *
 * This file contains the implementation of the device parameter database.
 *
 * This file is part of Max data acquisition software
 * Copyright (C) 1998 Christian Rosen
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

//list class stores the device parameters which can be seen and edited in
//the listbox
//it also provides all functions to access it -> see header file

#include <iostream.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

#include "list.hh"
#include "error.hh"
#include "packet.h"

void List::initialize() {
}

//insert int var at beginning of list
//insert does not set yscale and offset -> insert that when you need it!
void List::insert(int i_var,char *dev_name,char *var_name,int save_flag, int type) {
  Element *ZT = new Element(i_var,dev_name,var_name, save_flag, type, 0,0,L);
  assert(ZT != 0);
  L = ZT;
}

//insert does not set yscale and offset -> insert that when you need it!
//insert double var at beginning of list
void List::insert(double d_var,char *dev_name,char *var_name,int save_flag,int type) {
  Element *ZT = new Element(d_var,dev_name,var_name, save_flag, type, 0,0,L);
  assert(ZT != 0);
  L = ZT;
}

// function returns pointer to last element of the list
Element *List::End_Of_List() {
  if ( L == 0 )
    return 0;

  Element *Prev,*Act;
  Prev = Act = L;
  while (Act) {
    Prev = Act;
    Act = Act -> next;
  }
  return Prev;
}

//this function checks wether device_name is already in the list
//and returns a pointer to it.
//it returns NULL if not
class Element * List::exist(char *dev_name, char *var_name) {
  Element *ListPtr; //Pointer to list element
  ListPtr = L;      //set ptr to start of list
  while (ListPtr) {
    if ( !strcmp(dev_name,ListPtr -> device_name.c_str())  &&
		 !strcmp(var_name,ListPtr -> var_name.c_str()))
      return ListPtr; // element found
    ListPtr = ListPtr -> next;
    }
  return ListPtr;  //return NULL if not found
}

void List::append(int i_var,char *dev_name,char *var_name,int save_flag,
		  int var_type,int scale,int offset) {
  Element *ListPtr;
  //first check, if Element is not yet present
  if ( (ListPtr = exist(dev_name,var_name) ) != NULL ) {
    //printf("List::append : element %20s %20s is already in the list \n",dev_name,var_name);
    ListPtr -> enabled = save_flag; //update enabled status
       ListPtr -> i_var = i_var; //replace existing value
    return;
  }
  Element *ZT = new Element(i_var,dev_name,var_name,save_flag, var_type,
			    scale,offset);
  //only one argument L is optional
  assert(ZT != 0);
  if ( L == 0 ) { //list is empty
    L = ZT;
    return;
  }
  Element *End = End_Of_List();
  End->next = ZT; //append new element to list
}

void List::append(double d_var,char *dev_name,char *var_name,int save_flag,
		  int var_type,int scale,int offset ) {
  Element *ListPtr;

  if ( (ListPtr = exist(dev_name,var_name) ) != NULL ) {
    //    printf("List::append (double) element %20s %20s is already in the list \n",dev_name,var_name);
    ListPtr -> enabled = save_flag; //update enabled status
    ListPtr -> d_var = d_var; //replace existing value
    return;
  }

  Element *ZT = new Element(d_var,dev_name,var_name,save_flag, var_type,
			    scale,offset ); //only one arg. L is optional
  assert(ZT != 0);
  if ( L == 0 ) { //list is empty
    L = ZT;
    return;
  }
  Element *End = End_Of_List();
  End->next = ZT;
}

//following functions set the variable value of an element in the list defined by it's
//position, used for device data listbox editor
void List::putvar(int value,int position) {
 Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
  while (ZT && pos++ < position)
     ZT=ZT->next;
  ZT->i_var = value;
}

void List::putvar(double value,int position) {
 Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
  while (ZT && pos++ < position)
     ZT=ZT->next;
  ZT->d_var = value;
}

void List::putvar(char *string,int position) {
 Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
  while (ZT && pos++ < position)
     ZT=ZT->next;
  strcpy(ZT->s_var,string);
}

//the following functions are used for the listbox feature
//they give info about elements defined by their positions in the list
int List::get_type(int position) {
  Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
  while (ZT && pos++ < position)
     ZT=ZT->next;
  if (!ZT)
    return -1;
  return ZT->type;
}

int List::get_store_flag(int position) {
  Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
  while (ZT && pos++ < position)
     ZT=ZT->next;
  if (!ZT)
    return -1;
  return ZT->storeflag;
}

int List::get_plot_flag(int position) {
  Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
  while (ZT && pos++ < position)
     ZT=ZT->next;
  if (!ZT)
    return -1; //end of list reached, value not found, index too high
  return ZT->enabled;//if zero, variable is not to be plotted
}

//identical to get_plot_flag
int List::get_enabled_flag(int position) {
  Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
  while (ZT && pos++ < position)
     ZT=ZT->next;
  if (!ZT)
    return -1; //end of list reached, value not found, index too high
  return ZT->enabled;//if zero, variable is not to be plotted
}

void List::set_plot_flag(int position,bool state) {
  Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
  while (ZT && pos++ < position)
     ZT=ZT->next;
  if (ZT)
    ZT->plotflag=state;
}

void List::set_enabled_flag(int position,bool state) {
  Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
  while (ZT && pos++ < position)
     ZT=ZT->next;
  if (ZT)
    ZT ->enabled=state;
}

//return device name as function of position in List
char *List::get_name(int position) {
Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
 while (ZT && pos++ < position)
     ZT=ZT->next;
 if (!ZT)
   return NULL;
 return (char *) ZT->device_name.c_str();//if zero, variable is not to be plotted
}

//return the variable name of device at position i
//used for the Init Plotwidgetrotuine
char *List::get_var_name(int position) {
Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
 while (ZT && pos++ < position)
     ZT=ZT->next;
 if (!ZT)
   return NULL;
return (char *) ZT -> var_name.c_str();
}

//function returns the int value of the type flag of device
char* List::get_data_type(char *dev_name) {
  char *typelist;
  typelist = new char[1000];
  assert (typelist != 0);
  Buffer_pointer = L;
  int pos;
  //scroll through list until first element of device_name is found
  for (pos = 0; pos < 1000; pos ++)
    typelist[pos]='\0';
  pos = 0;
  while (Buffer_pointer) {
    if (strcmp(dev_name,Buffer_pointer->device_name.c_str())==0) {
      switch  (Buffer_pointer -> enabled) {
      case 1 : //int
	typelist[pos++]='i';
	break;
      case 2 :
	typelist[pos++]='d';
	break;
      case 3 :
	typelist[pos++]='c';
	break;
      case 4 :
	typelist[pos++]='s';
	break;
      }
    }
  Buffer_pointer = Buffer_pointer -> next; //go to next element
  }
  typelist[pos]='\0';
return typelist;
}

void List::reset_data_buffer() {
  BufferPointer = L;
}

void List::ResetHelpPointer() {
  BufferPointer = L;
}

//set secondary pointer to next element in the lsit
//return zero when @ end of list This is a cheap iterator simulation
int List::AdvanceHelpPointer() {
  if (BufferPointer -> next) {
    BufferPointer = BufferPointer -> next;
    return TRUE;
  }
  else
    return FALSE;

}

//return sevice data of element <device_name>
//in packet structure for socket transfer
//used for sending devData to polld
devDataPacket *List::GetPacketElement(char *dev_name) {
  Element *ListPtr;
  ListPtr = L;     //set pointer to
  memset(Packet,0,sizeof(devDataPacket));
  char *buffer;
  buffer = new char[MAX_NAME_LENGTH+MAX_VAR_LENGTH]; //allocate mem for strings and clear it
  while (ListPtr) {
    if (strcmp(dev_name,BufferPointer -> device_name.c_str()) == 0) {
      strncpy((char *) Packet ->device_name,
	      BufferPointer -> device_name.c_str(),MAX_DEVICE_NAME_LENGTH);
      strncpy((char *) Packet -> var_name  ,
	      BufferPointer -> var_name.c_str()   ,MAX_NAME_LENGTH);
      Packet -> enabled = (unsigned char) BufferPointer -> enabled;
      switch(BufferPointer -> type){ //check for variable type
      case 0:
	break;
      case 1://int
	sprintf( buffer," %d",BufferPointer -> i_var);
	strncpy(( char *) Packet -> value, buffer,MAX_VAR_LENGTH);
	Packet -> type = (char ) BufferPointer -> type;
	break;
      case 2: //double
	sprintf( buffer,"%G",BufferPointer -> d_var);
	strncpy(( char *) Packet -> value, buffer,MAX_VAR_LENGTH);
	Packet -> type = (char ) BufferPointer -> type;
	break;
      case 3: //char;
	printf("ERROR : List::GetPacketElement : no char variable !!\n");
	break;
      case 4: //string
	printf("ERROR : List::GetPacketElement : no string variable !!\n");
	break;
      default :
	break;
      }
    }
    ListPtr = ListPtr -> next;
  }
  delete buffer;
  return Packet;
}

//return packet structure with elements
//pointed to by the secondary help pointer
//used for sending devData to polld
devDataPacket *List::GetNextPacket(void ) {
  memset(Packet,0,sizeof(devDataPacket));
  assert (Packet !=0 );
  char *buffer;
  buffer = new char[MAX_NAME_LENGTH+MAX_VAR_LENGTH];        //allocate mem for strings and clear it
  if (BufferPointer) {
    strncpy((char *) Packet ->device_name,
	    BufferPointer -> device_name.c_str(),MAX_DEVICE_NAME_LENGTH);
    strncpy((char *) Packet -> var_name,
	    BufferPointer -> var_name.c_str(),MAX_NAME_LENGTH);
    Packet -> enabled = (char ) BufferPointer -> enabled; //copy enabled flag
    switch(BufferPointer -> type){ //check for variable type
    case 0:
      break;
    case 1://int
      sprintf( buffer," %d",BufferPointer -> i_var);
      strncpy(( char *) Packet -> value, buffer,MAX_VAR_LENGTH);
      Packet -> type = '1';
      break;
    case 2: //double
      sprintf( buffer,"%G",BufferPointer -> d_var);
      strncpy(( char *) Packet -> value, buffer,MAX_VAR_LENGTH);
      Packet -> type = '2';//(char ) BufferPointer -> type;
      break;
    case 3: //char;
      printf("ERROR : List::GetPacketElement : no char variable !!\n");
      break;
    case 4: //string
      printf("ERROR : List::GetPacketElement : no string variable !!\n");
      break;
    default :
      break;
    }
  }
  delete buffer;	
  return Packet;
}

// Access the Device Parameter Database to retreive a int value
int List::getvar(int *i_var,char *dev_name,char *var_name) {
  *i_var =0;
  Element *ZT = L ;
  while ( ZT && varcomp(ZT, dev_name,var_name))
    ZT=ZT->next;
  if (!ZT) {
    printf("Polld in function : List::getvar(int *) : element %s not found\n",var_name);
    return 0;
  }
  *i_var=ZT->i_var;
  return 1;
}

int List::getvar(double *d_var,char *dev_name,char *var_name) {
  *d_var =0;
  Element *ZT = L ;
  while ( ZT && varcomp(ZT, dev_name,var_name))
    ZT=ZT->next;
  if (!ZT) {
    printf("Polld in function : List::getvar(double *) element %s not found\n",var_name);
    return 0;
  }
  *d_var=ZT->d_var;
  return 1;
}

int List::putvar(int i_var,char *dev_name,char *var_name) {
  Element *ZT = L ;
  while ( ZT && varcomp(ZT, dev_name,var_name))
    ZT=ZT->next;
  if (!ZT) {
    printf("Polld in function : List::putvar(int) : element %s not found!\n",var_name);
    return 0;
  }
  ZT->i_var = i_var;
  return 1;
}

int List::putvar(double d_var,char *dev_name,char *var_name) {
 Element *ZT = L ;
  while ( ZT && varcomp(ZT, dev_name,var_name))
    ZT=ZT->next;
  if (!ZT) {
    printf("List::putvar(double) : element %s not found!!!\n",var_name);
    return 0;
  }
  ZT->d_var = d_var;
  return 1;
}

//test the status of a "channel' of the device
//return true if the enabled flag is set
//e.g. if the user has switched on this "channel"
int List::enabled(char *dev_name,char *var_name) {
  Element *ZT = L ;
  while ( ZT && varcomp(ZT, dev_name,var_name)==-1)
    ZT=ZT->next;
  if (!ZT) {
    printf("Polld in function : List::enabled : element %s not found!!!\n",var_name);
    return 0;
  }
  return ZT -> enabled;
}

//return 0 : no store-variable
//return 1 : store variable, not enabled
//       2 : store var enabled
//return 3 : store var enabled and plot
int List::plotstatus(int position) {
  int     k  = 1;               //start to count from 1 and not from zero
  Element *ZT = L;
  while (ZT && k++ < position) {
    ZT = ZT -> next;     //scroll through list to position
  }
  if (!ZT)
    return -1;
  if (ZT -> enabled == -1)
    return 0;
  if (ZT -> plotflag && ZT -> enabled )
    return 3;
  return -1;
}

//removes complete list out of memory
void List::remove() {
  Element *ZT = L;
  while (ZT ) {
    Element *tmp = ZT;
    ZT= ZT->next;
    delete tmp;
  }
  L  = 0;
}

//this function tells you wether the specified device is an active device
int List::is_active(char *dev_name) {
  Element *ZT = L;
  while ( ZT && varcomp(ZT, dev_name,"active"))
    ZT = ZT -> next;
  if (!ZT)
    return -1;
  return 1;
}

void List::printlist() {
  printf( "\t(\n");
  for (Element *ZT = L ; ZT ; ZT = ZT -> next) {
    if (ZT -> type == 1 )
      printf("\t\t%d %s  %s, \n",ZT -> i_var, ZT->device_name.c_str(),ZT->var_name.c_str() );
    else
       printf("\t\t%f %s  %s, \n",ZT -> d_var, ZT->device_name.c_str(),ZT->var_name.c_str() );
  }
  printf("\t)\n");
}

//help functions, don't belong in this file
int List::varcomp(Element *currentPtr,char *device_name, char *var_name) {
  x=strcmp(var_name,currentPtr->var_name.c_str());
  y=strcmp(device_name,currentPtr->device_name.c_str());
  if (!x && !y)
    return 0;
  else
    return -1;
}
