/**************************************************************************
 * $Id: list.hpp,v 1.2 1999/02/21 18:48:41 rosen Exp rosen $
 *
 * Revision 1.01 1998/10/03 14:34:15  rosen
 *
 * This file is part of Max data acquisition software
 * Copyright (C) 1998 Christian Rosen
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **************************************************************************/

/*! \file list.h
 *\brief This file contains the list class for the polld deamon ( The server)  and user devices to handle the \e device \e parameter \e database.
 *This class stores the device parameters which can be seen and edited
 *in the listbox of the MainGUI.
 *It also provides all functions to access this data. The device-modules use the routines :
 * int List::getvar(int *,char *, char *) from this class
 *
 *The list header file contains the class declaration for the List class.
 * \author C. Rosen
 */
//! \dontinclude laser.cpp

#ifndef LIST_H
#define LIST_H

#include "packet.h"

#include <iostream.h>
#include <assert.h>
#include <string.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

//STL includes
#include <string>

#include <qstring.h>
#include <qlist.h>


//the data_type flag contains the variable type of the save variable
// -> see : List::get_data_type(char *)
//this is neccessary to determine the type for saving the data
//Scheduler::store(var) now sets up an array during first measurement cycle to store data types
//look there to figure that out
//IMPROVEMENT :
//BUT ADD TEMPLATE LIST CLASS TO REMOVE SWITCH STATEMENTS IN GETSTRING AND
// GETALLDEVICEPARAMETERS functions
//
/*class Element
 *\brief It contains one element of the linked list of device parameters.
 */
class Element {
 friend class List;
public:

private:
//private Konstructor for int var
  Element(int W, const char *device, const char *var,int flag,int data_type,
	int x=0,int y=0, Element *E = 0) {
    i_var       = W;
    next        = E;
    enabled   = flag; //this is the enabled flag !!!! TRUE -> device stores the data
    plotflag    = y;   //TRUE is var is plotted
    type        = data_type;
    storeflag   = x;   //TRUE if variable is a store variable
    device_name = device;
    var_name    = var;
  }
//private Konstructor for double var
 Element(double W, const char *device,const char *var,int flag, int data_type,
	int x=0,int y=0, Element *E = 0) {
    d_var       = W;
    next        = E;
    enabled     = flag; //this is the enabled flag
    plotflag    = y;    //plot ot ?
    type        = data_type;
    storeflag   = x;  //indicates that thiselemet is a "store" variable
    device_name = device;
    var_name    = var;
 }	
  ~Element() {
    device_name.~string();
    var_name.~string();
  }
  string   device_name;
  string   var_name;     //data ist a reserved word for a variable name
  int      enabled;    //if that is nonzero the according variable
  //is saved to disk and can be plotted
  //1 = int,2 = double, 3 =char, 4 = string
  int      i_var;
  double   d_var;
  char     c_var;
  char *s_var;
  Element  *next;
  int       type;      //type flag 1 = int,2 = double, 3 = char, 4 = string
  int       storeflag; //TRUE is var is a store variable
  int       plotflag;  //TRUE is variable is to be plotted
};

/*!\class List list.h
 *\brief This class represents the list of the \e Device \e Parameter \e Data together with the functions to access it.
 * Check here for int List::putvar(int, char *,char *), int List::getvar(int *,char *,char *) and int List::enabled(char *,char *) functions.
 *
 *The \e Device \e Parameter \e Data is the data that is given in the \e initfile, which is read by \b MAX at startup. This data can be edited in the left listbox of the MainGUI.
 *Each device parameter is uniquely defined by two arguments.
 *A \e Device-Name and a \e Variable-Name.
 *These are always given as arguments when accessing the value.
 *See also the default initfile \e "initfile" on how to set up a database.
 *Only the class-functions that are important for the user are documented.
 *You may not want to call the other functions.
 *A documentation for the rest can be found in the source code directly. (see : list.cpp, list.cpp)
 */

class List
{
public:
  List()
  {
    L=0; Buffer_pointer=L;
    ParameterStr = new char[10000];
    assert(ParameterStr !=0);
    Packet= new devDataPacket;
  }
  ~List() {
    remove();
  }
  void initialize();
  void insert  (int   , char *,char *,int,int ); //insert new data-element
  void insert  (double, char *,char *,int,int ); //dito but with double
  void append  (int,    char *,char *,int,int,int,int ); //append new data-element (at the end)
  void append  (double ,char *,char *,int,int,int,int ); //dito but with double-variable
  Element * exist(char *,char *);                //check for deviceName && varName in the list
  void remove  ();                        //removes complete list from memory
  int is_active(char *);                  //returns 1 is char* device_name is an active device,
                                          //otherwise -1
  int    getvar (int *   ,char *,char *); //first  device then var-name
  /*! \fn int List::getvar(int *   ,char *,char *)
   * This function is called from the device module to retrieve a device Parameter from the
   * Device Parameter Database.
   *
   *  \e An \e Example from the the file laser.cpp  :
   *
   * How to read the variable "laser", "phase" into the integer value b :
   *
   * \line enabled result = device_data ->getvar(&b,"laser","phase")
   * \line }
   * \param Value, a reference to the \e integer variable to be received
   * \param Device_Name,  the device name
   * \param Variable_Name,  the variable name
   * \return -1 if variable was not found(argument is zero),
   * \reuturn 1 if variable was found
   */
  int    getvar (double *,char *,char *); //same for double vars
  /*! \fn int List::getvar(double *   ,char *,char *)
   * This function is called from the device module to retrieve a device Parameter from the
   * Device Parameter Database.
   * \param Value, a reference to the \e double variable to be received
   * \param Device_Name,  the device name
   * \param Variable_Name,  the variable name
   * \return -1 if variable was not found(argument is zero), 1 if variable was found
   */
  int    putvar (int     ,char *,char *); //set variable value (int)
  /*! \fn int List::putvar(int *   ,char *,char *)
   * This function is called from the device module to store a device Parameter into the
   * Device Parameter Database.
   * \param Value, the \e integer variable to be stored
   * \param Device_Name,  the device name
   * \param Variable_Name,  the variable name
   * \return -1 if variable was not found,
   * \return 1 if variable was found.
   */
  int    putvar (double  ,char *,char *); //set variable value (double)
 /*! \fn int List::putvar(double *   ,char *,char *)
   * This function is called from the device module to store a device Parameter into the
   * Device Parameter Database.
   * \param Value, the \e double variable to be stored
   * \param Device_Name,  the device name
   * \param Variable_Name,  the variable
   * \return 1 if variable was found,
   * \return 0 if not
   */
  int    varcomp(Element *,char *,char *);
  void   putvar(int, int);                //set new variable for element at position i
  void   putvar(double, int);
  void   putvar( char *,int);
  int    enabled(char *,char *);          //return TRUE if this var is enabled
  /*! \fn int List::enabled(char *,char *)
   * This function is very important! It checks the \c ENABLED field of the \c STORE
   *section in the \e initfile, which is read by \b MAX at startup.
   *The device_module checks via this function, wether a specific channel is \e "enabled",
   * This way the device knows, if it should take data on this channel or not.
   * (See also Int Scheduler::store(int ) ) , the file laser.cpp and the descrption of the initfile
   * \param Device_Name, the name of the device
   * \param Variable_Name, the name of the \c STORE variable to check
   * \return TRUE if this "channel" is enabled,
   * \return FALSE if not
   */
  int    plotstatus(int );                //return plotstaus(nostorevar,enabled,plot)
  char*  get_data_type(char *);           /* returns a string with a list of the
	 *saved data type of device char
	 *f.e. : the variables string,double,int
	 *would return : sdi   *
	 * needed for the save-routine*/
  /*network related functions only in ver 2.0 !!!*/
  devDataPacket * GetPacketElement(char *); //return struct packet for socket transfer
  devDataPacket * GetNextPacket(void );   //return data element pointed to by BufferPointer
  void   ResetHelpPointer(void);         //reset help list pointer to beginning of list
  int    AdvanceHelpPointer(void);       //go to next element
  /* end */
  void   reset_data_buffer();             //which is needed for parameter file save routine
  void   printlist();
  Element* End_Of_List();
  char *  getstring(int);                  //returns string with dev-name par-name value
  int    get_type(int );	      	  //returns type of ith element		
  char * get_name(int );                  //return name of device at position i
  char * get_var_name(int );              //return var name of device at position i
  int    get_store_flag(int );            //return the storeflag out of parameter database
  int    get_plot_flag(int);      //check if variable specified by position contains plotdata (ysize)
  int    get_enabled_flag(int);
  void   set_enabled_flag(int,bool);        //toggle the flags according to user input
  void   set_plot_flag(int,bool);         //"switch" on/off a variable
  int    get_number_plotvars(char *);     //return number of variables to be plotted
  string get_plotvar_namelist(char *);   //defined by "data" statement in initfile
                                          //or zero if list is empty or at the end
private:
  char *ParameterStr;
  devDataPacket *Packet;
  int index;    //index for plotvariable
  int x,i,y;
  Element *L;
  Element *BufferPointer,*Buffer_pointer;
};

#endif

//Don't write behind the endif line !!

