/*
 * MAX : polld.h, Sam Dez 29 20:03:10 CET 2001 -sg
 *
 * polling deamon main programm header file
 *
 * This file is part of Max data acquisition software
 * Copyright (C) 1998 Christian Rosen
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

//connectDevices_putvar routine with siognal to stack packet
#ifndef _POLLD_H_
#define _POLLD_H_

#include "definitions.h"
//#include "qtincludes.h"
#include "packet.h"
#include "network.h"
#include "list.hh"
#include "sched.hh"
#include "priority.h"
//qt includes

#include <qapp.h>
#include <qlist.h>
#include <qstring.h>
#include <qsocketnotifier.h>

//solaris includes
//#include <sys/filio.h>
#include <unistd.h>
#include <netdb.h>
#include <netinet/in.h>

//main class for polling deamon
class Polld : public QApplication
{
    friend class Scheduler;
    Q_OBJECT;
public :
    Polld(int argc,char* argv[]);
    ~Polld();
protected:
  int socketConnect(char * ); //Connect all Sockets
  void stackData();
  void initialize(void);        //reset and clear all variables and databases
  void emptyDevDataQueue(void); //add received devData top database
  void sendDevData(void );      //send device Parameters back to max GUI
  devDataPacket * makeDevPacket (char *,char *,int   );
  devDataPacket * makeDevPacket (char *,char *,double);
signals :
//  void  Sig_Error(char *,char *);
  void  start();
  void Timeout();
protected slots :
  int  sendCommand(char ,char *,int ,char *);
  void sendParameters(); //send act. int and channel number to GUI
  //void sendDevParameter(*devDataPacket); //send one set of device parameters
  void sendData();
  void commandReceived();
  void dataReceived();
  void startMeasurement();
  void TriggerTimerEvent(); //measurement routine trigered internally
  void SendError( char * ); //send Error message to max
private:
  /* first, the network stuff */
  //	 Queues for stacking the data in between two send intervalls
  QList<	devDataPacket> *DevSQueue; //device packet queue for sending
  QList<devDataPacket> *DevRQueue; //device packet queue for receiving

  devDataPacket      *d_packet;   //!Packet for sending dev_parameters
  command_packet     c_packet;   //!Packet for sending cmd and measurement infos
  devDataPacket      *r_d_packet; //!packet to receive device data
  dataPacket         dat_packet; //!Packet for sending measured data
  QSocketNotifier    *CommandNotifier;
  QSocketNotifier    *DataReceivedNotifier;
  int                sockD;
  int                data_sfd;      //socket-filedescriptor for data socket
  int                send_data_sfd;  //  "         "      for sending DevData
  int                cmd_sfd;       // "         "          for command sockets
  int                cmdr_sfd;      //command receive socket filedescriptor
  int                par_sfd;       // "         "        for device parameters
  int                mdata_sfd;    //send measured data socket
  struct hostent     *hp;
  struct hostent     *host,hostBackup;
  struct sockaddr_in server;        //polldeamon is the server
  struct sockaddr_in server_max;    //structure to identify max as server
  struct sockaddr_in client;        //data recieved from max
  struct sockaddr_in client_pd;     //client structure for polld-client
  struct sockaddr_in cmdClient_max; //client who receives commands
  struct sockaddr_in cmdClient_pd;  //cleint who sends commands
  struct sockaddr_in cmdServer_max;
  struct sockaddr_in cmdServer_pd;
  struct sockaddr_in dataServer;    // polld is only server for maxeasured data
  struct sockaddr_in dataClient;   //structure for receiver of dataClient
  struct sockaddr    name;
  int                sockd;
  int                flag;
  int	  	     a_sfd; //port
  int		     b_sfd; //port+1
  int  		     c_sfd; //port+2
  int		     d_sfd; //port+3
  int		     e_sfd; //port+4
  unsigned int       server_len;
  unsigned int       client_len;
  unsigned int       client_pd_len;
  unsigned int       cmdClient_max_len; //length of received command packet
  unsigned int       cmdClient_pd_len;
  unsigned int       cmdServer_max_len; //length of com.packets send
  unsigned int       dataServer_max_len;      //length of send Data structure

  sockaddr_in  server_addr; //adress of the server
  sockaddr_in  data_send;   //adress structure for data send connection
  sockaddr_in  cmd_client_addr;
  sockaddr_in  cmd_server_addr;
  sockaddr_in  cmd_send;   //adress strucutre for command send connection
  sockaddr_in  cmd_receive;//adress structure for command receive connection
  sockaddr_in  par_send;   //adress structure for device parameters send connection
  sockaddr_in  par_receive;//adress structure for device parameters receive connection
  sockaddr_in  par_server_max_addr;
  sockaddr_in  par_server_pd_addr;
  sockaddr_in  par_client_max_addr;
  sockaddr_in  par_client_pd_addr;
  sockaddr_in  cmd_server_pd_addr;
  sockaddr_in  cmd_client_pd_addr;
  sockaddr_in  cmd_server_max_addr;
  sockaddr_in  cmd_client_max_addr;
  unsigned int data_send_len;
  unsigned int cmd_server_addr_len;
  unsigned int cmd_server_max_addr_len;
  unsigned int cmd_server_pd_addr_len;
  unsigned int cmd_client_max_addr_len;
  unsigned int cmd_client_pd_addr_len;
  unsigned int par_server_pd_addr_len;
  unsigned int par_client_max_addr_len;
  unsigned int par_server_max_addr_len;
  unsigned int par_client_pd_addr_len;

  int     main_sfd     ;     //main socket file descriptor
  int     cmd_main_sfd ;
  int     cmd_main_sfd2;
  int     par_main_sfd ;
  int 	par_main_sfd2  ;


  /*end of network variables*/
  int               orig_sfd,orig_sfd1,orig_sfd2,o_sfd1,o_sfd2;
  List               *Devices;      //class for device database
  Pr_list            Priority;     //list of priorities
  Scheduler          *Schedule;     //scheduler class for routines
  /* flags that control the measurement */
  int                data_send_interval;
  int                stopflag;     	// If TRUE Measurement stops
  int                pause_flag;   	 // if TRUE   " measurement pauses   "
  int                save_flag;    	//TRUE : database holds unsaved data
  int                m_running;    	// TRUE == Measurement runs
  int                continue_flag; 	//to enable continue button
  int                n;
  int                triggermode; // 1 = external, 0 = internal
  int                monitor;
  int                trigger_period;
  int                act_channel,max_channels;
  int                act_int,max_integrations;
  int                send_period;  //intervall for sending data (chan numbers)
  bool               readytosend;  //SendDatatimer has run out-> sendData
  int                wait;
  int                cmdvalue;     //value of command sequence parameter
  bool 		     suspend;
  char               cmdname[MAX_NAME_LENGTH]; //text of command sequence
  char *buf[BUFSIZE];
  /* QT related stuff */
  QTimer             *intTrigger;
  QTimer             *sendMData;
  QTimer             *send1;
  QString            *str;  //a string to convert the data to char
};

#endif

