/*
 * MAX : priority.cc, Sam Dez 29 20:04:09 CET 2001 -sg
 *
 * this file contains the implementation of the priority_list it list 
 * contains the order of the devices called during the measurement
 *
 * This file is part of Max data acquisition software
 * Copyright (C) 1998 Christian Rosen
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include <stdio.h>
#include <string.h>

#include "priority.h"
#include "definitions.h"

#undef DEBUG

//remove complete list out of memory
void Pr_list::Pr_remove() {
	Pr_element *ZT=L;
  	while (ZT) {
    		Pr_element *tmp = ZT;
    		ZT = ZT ->next;
    		delete tmp;
  	}
  	L = 0;
}

int Pr_list::Pr_empty() {
  	return L == 0 ? 1 : 0 ;
}

int Pr_list::N_Devices() {
  	Pr_element *ZT = L;
  	int length=0;
  	while (ZT) {
    		ZT = ZT -> next;
    		length++;
  	}
  	return length;
} 		

//initlist(char *) :
//this routine reads an initfile from disk that contains the priority list
//it has the following form  : <device_name> <priority>
//priority is an integer number from 0 (device off) to DEVICE_NUMBER (last device called)
int Pr_list::Pr_initlist(char *filename) {
  	char device_name[256];
  	char comment[300];
  	int priority;
  	FILE *fp;
  	if (( fp = fopen(filename,"r")) ==NULL) {
    		printf("Could not open priority file !\n");
    		QMessageBox::query("ATTENTION!!",
		       "Priority File not found.\n I will keep the old data","OK");
    		return 0;
  	}	
  	Pr_remove();

#ifdef DEBUG
  	printf("\"priority\":\n");
#endif
  
  	while (fscanf(fp,"%s",device_name) != EOF) {
     		if (strncmp(device_name,"#",1)==0) {       //check for comment line
           		fgets(comment,300,fp);       	//read until end of line
     		}
     		else {
       			if (strlen(device_name) > MAX_DEVICE_NAME_LENGTH ) {
	 			printf("\n\n ******************************\n");
	 			printf(" *   Priority-init : ERROR    *\n");
	 			printf(" ******************************\n");
	 			printf(" Device name %s \n is too long !!\n", device_name);
				printf(" Maximum length of device_names is : %d\n",
						MAX_DEVICE_NAME_LENGTH);
				printf(" To Change the maximum length, alter the 'MAX_DEVICE_NAME_LENGTH'
\nconstant in the file 'definitions.hpp' \nand recompile by typing 'make clean' and 'make'.\n\n");
				return 0;
       			}
       			if (fscanf(fp,"%d",&priority)==0) {
	 			printf("\n**********************************************************\n");
	 			printf("ERROR in Priority Initfile : '%s'!!\n",filename);
	 			printf("Argument given after device '%s' is not a number ! \n",device_name);
	 			printf("Check the Priority initfile\n");
	 			printf("This Application will terminate now,if this\n");
	 			printf("is the first initialization of Max..\n");
	 			printf("In case you have justed reloaded a new file :\n");
	 			printf("%sThe device and Priority database may be corrupt now!\n","\033[5m");
	 			printf("You have to restart Max, or reload a correct database\n");
	 			printf("%sbefore starting a new measurement !\n","\033[25m");
	 			printf("**********************************************************\n");
	 			return 0;
       			}
#ifdef DEBUG
       			printf("\t%-20s : %-2d \n",device_name,priority);
#endif
       			Pr_append(device_name,priority); //append data to priority list
     		}
  	}
	return 1;
}

Pr_element *Pr_list::end() {
    if (L == 0)
      return 0;//List is empty

  Pr_element *prev,*act;
  prev = act = L;
  while ( act ){
    prev = act;
    act = act -> next;
  }
  return prev;
}

void Pr_list::Pr_append(char *dev_name,int priority) {
  // new (6/98) : important for polld deamon, when receiving data
  //first check,if element is not yet present
  //if present just replace priority for the device
  Pr_element *PrPointer;
  if ( (PrPointer = exist(dev_name) ) != NULL ) {
    //    printf("Priority : %s already in PrList \n",dev_name);
    PrPointer -> priority = priority;
    return;
  }
  //if not, append element
  Pr_element *el = new Pr_element(dev_name, priority);
  assert(el != 0 );
  if ( L == 0) {
    L=el;
    return;
  }
  Pr_element *last = end();
  last->next = el;
}

//this routine checks, wether dev_name already exists
// in the priority list
Pr_element *Pr_list::exist(char *dev_name) {
  Pr_element *PrPointer;
  PrPointer = L;
  while (PrPointer) {
    if ( !strcmp(dev_name,PrPointer -> device_name) )
      return PrPointer;
    PrPointer = PrPointer -> next;
  }
  return PrPointer;
}

//this routine returns true if the priority pr already exists
int Pr_list::exist( int pr) {
  if (L == 0) return 0;
  Pr_element *Act=L;
  while (Act) {
    if (Act -> priority == pr)
      return 1;
    Act = Act -> next;
  }
  return 0;
}

void Pr_list::Pr_kill_list() { //kill the complete list
  Pr_element *el = L;
  while (el) {
    Pr_element *tmp = el;
    el=el->next;
    delete tmp;
  }
  L  = 0;
}

//get priority of device defined by dev_name
int Pr_list::Get_priority(char *dev_name) {
  Pr_element *el=L;
  while ( el && varcomp(el,dev_name))
    el=el->next;
  if (!el) {
    //printf("priority : element %s not found\n",dev_name);
    return -1;
  }
  return el->priority;
}

//return priority of device at position i in list
int Pr_list::Get_priority(int i) {
  Pr_element *el=L;
  int x = 0;
  if ( i == 0)
    return el -> priority;
  while (el && x++ < i)
    el=el->next;
  if (!el) {
    //printf("get priority : not so many elements!!!!!\n");
    return -1;
  }
  return el->priority;
}

void Pr_list::Put_priority(int priority,int index) {
  Pr_element *el = L;
  int x= 0;
  if ((index) == 0) {
    el -> priority = priority;
    return;
  }
  while (el && x++ < index)
    el = el -> next;
  if (!el) {
    return ;
  }
  el -> priority = priority;
}

void Pr_list::Sort_priority() { //!this function doesn't work yet !!!
  int counter = 0;
  int elements= 0;
  int pos=0;;
  if (!L ) return;
  Pr_element *PrPointer;
  PrPointer = L;
  while (PrPointer) {  //count number of elements
    elements ++;
  }
  printf("length of priolist %d \n",elements);
  while(counter != elements) {
    counter ++;
    if (pos== exist(counter) )
      counter++;        //this prio exists, check next one
    else { //this priority is missing reset priorities

    }	
  }
}

//function returns the name of device at the current postion in the
//priority list. needed to check foir active status of device at position in device_name list
//for this you need to get the device name (this is a little complicated...)
char * Pr_list::Get_device_name(int position) {
 Pr_element *el=L;
  int x = 0;

  while (el && x++ < position)
    el=el->next;
  if (!el) {
    //    printf("priority : not that many elements!!!!!\n");
    return NULL;
  }
  return el->device_name;
}

//returns device name of device with prioirity prior
char * Pr_list::Get_dev_name(int prior) {
  Pr_element *el = L;
  while (el && prior != el->priority)
    el = el-> next;
  if (!el)
    return NULL;
  return el->device_name;
}

//simply print the whole list (for debugging purposes)
void Pr_list::Pr_printlist() {
  for (Pr_element *Act = L; Act ; Act = Act -> next )
    printf("device : %20s priority : %3d \n",Act -> device_name,Act -> priority);
}

//functions return zero if at least one priority is missing in the list
int Pr_list::Consistency() {
  int act_prio = 0;
  int max_prio = Max_Priority();
  int y=0;
  if (Max_Priority() > N_Devices() )
    return 0;
   while (act_prio < max_prio ) {
    act_prio++;
    if (exist(act_prio))
      y++;
  }
  if (y < max_prio)
    return 0;
  return 1;
}

//Return the maximum priority in the priority list
int Pr_list::Max_Priority() {
  int y;
  int maximum=0;
  for (y=0;y< N_Devices();y++) {
  if (maximum < Get_priority(y) )
      maximum = Get_priority(y);
  }
return maximum;
}

int Pr_list::check4double() {  //check for double priorities
  //  printf("check4double %d\n",N_Devices());
  bool flag=FALSE;
  int x,y;
  for (x=1;x <= N_Devices();x++) {   //loop over priorities
    flag = FALSE;
    for (y=0;y < N_Devices();y++) {    //loop over number of elements
      //  printf("outer loop x %d y %d \n",x,y);
      if (Get_priority(y)==x) {     //This priority exists don't check 0 priority
	//printf("inner loop x %d y %d \n",x,y);
	 if ( flag )
	   return 0;
	 else
	   flag = TRUE;
       }
     }
  }
  return 1;
}

//Check if all priorities are zero
//returns FALSE if all priorities are zero
bool Pr_list::check4zerolist() {
  bool flag = FALSE;
  int y;
  for (y=0;y < N_Devices();y++) {    //loop over number of elements
    if ( Get_priority(y) == 1 )
      flag = TRUE;
  }
  return flag;
}

//compare two strings and return 0 or -1
int Pr_list::varcomp(Pr_element *currentPtr,char *device_name)
{
  if (!strcmp(device_name,currentPtr->device_name))
    return 0;
  else
    return -1;
}

//the user can use a secondary pointer to the priority list
void Pr_list::ResetHelpPointer() {
  BufferPointer = L;
}

int Pr_list::AdvanceHelpPointer() {
  if (BufferPointer)
    BufferPointer = BufferPointer -> next;
  else
    return FALSE;
  return TRUE;
}

char *Pr_list::GetNextName() {
  if (BufferPointer)
    return BufferPointer -> device_name;
  return '\0';
}

int Pr_list::GetNextPriority() {
  if (BufferPointer)
    return BufferPointer -> priority;
  return -1;;
}
